// Bits is Bits
// Eric Mika, ITP Math Driveby, October 14 2010

// A brute-force example designed to emphasise the fact that the representational qualities of
// a particular string of data is a matter of interpretation. This example reinterprets image
// samples from a camera into a sound wave.

import processing.video.*;
import ddf.minim.*;
import ddf.minim.signals.*;

Capture cam;
Minim minim;
AudioOutput out;
SoundSignal soundSignal;

int sampleSize = 256;

void setup() {
  size(832, 240);

  // Set up the camera
  cam = new Capture(this, 320, 240);  

  // Set up the audo signal
  minim = new Minim(this);
  out = minim.getLineOut(Minim.STEREO, sampleSize);
  soundSignal = new SoundSignal();
  out.addSignal(soundSignal);
}

void draw() {
  // Read from the camera and draw into the background
  if (cam.available()) {
    cam.read();
    set(0, 0, cam);
  }

  // Paint over the waveform
  noSmooth();
  noStroke();
  fill(0);
  rect(cam.width, 0, 512, cam.height);

  // Draw the waveform
  noFill();  
  smooth();
  strokeWeight(2);  
  stroke(255);  
  beginShape();
  for (int i = 0; i < sampleSize; i++) {
    vertex(map(i, 0, sampleSize, 0, 512) + cam.width, map(out.right.get(i), 0, 1, 0, cam.height));
  }
  endShape();
  
  fill(255, 0, 0);
  text("Image to Sound", width - 120, 20);
}

class SoundSignal implements AudioSignal {
  // Fill the sound buffer with pixel data
  void generate(float[] samp) {
    for (int i = 0; i < sampleSize; i++) {
      samp[i] = abs(map(cam.pixels[round(map(i, 0, sampleSize, 0, 76800))], 0x000000, 0xFFFFFF, 0, 1));
    }
  }

  void generate(float[] left, float[] right) {
    generate(left);
    generate(right);
  }
}
