// Bits is Bits
// Eric Mika, ITP Math Driveby, October 14 2010

// A brute-force example designed to emphasise the fact that the representational qualities of
// a particular string of data is a matter of interpretation. This example reinterprets the text
// of Moby Dick into a sound wave.

import ddf.minim.*;
import ddf.minim.signals.*;

Minim minim;
AudioOutput out;
SoundSignal soundSignal;

int sampleSize = 512;
int readHead = 0;
byte[] mobyDickBytes;
String mobyDickSample = "loading";

void setup() {
  size(832, 240);

  // Load the book
  mobyDickBytes = loadBytes("moby-dick-ascii.txt");  

  // Set up the audo signal
  minim = new Minim(this);
  out = minim.getLineOut(Minim.STEREO, sampleSize);
  soundSignal = new SoundSignal();
  out.addSignal(soundSignal);
}

void draw() {
  background(0);
  
  // Draw the text
  fill(255);
  text(mobyDickSample, 0, 0, 320, 240);

  // Paint over the waveform
  noSmooth();
  noStroke();
  fill(0);
  rect(320, 0, 512, 240);

  // Draw the waveform
  noFill();  
  smooth();
  strokeWeight(2);  
  stroke(255);  
  beginShape();
  for (int i = 0; i < sampleSize; i++) {
    vertex(map(i, 0, sampleSize, 0, 512) + 320, map(out.right.get(i), 0, 1, 0, 240));
  }
  endShape();

  fill(255, 0, 0);
  text("Text to Sound", width - 100, height - 10);
}

class SoundSignal implements AudioSignal {
  // Fill the sound buffer with pixel data
  void generate(float[] samp) {
    // Load moby dick from the byte array
    mobyDickSample = "";

    int j = 0;
    for(int i = readHead; i < readHead + sampleSize; i++) {

      samp[j] = map(mobyDickBytes[i], 0, 127, -1, 1);
      mobyDickSample += char(mobyDickBytes[i]);
      j++;
    }

    // Wrap the read head once we reach the end
    readHead += sampleSize;  
    if(readHead > (mobyDickBytes.length - sampleSize)) readHead = 0;
  }

  void generate(float[] left, float[] right) {
    generate(left);
    generate(right);
  }
}

